@@ -1,3 +1,5 @@
+// SPDX-License-Identifier: Apache-2.0
+// 
 // Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
 // Copyright 2008-2016 National ICT Australia (NICTA)
 // 
@@ -49,12 +51,11 @@ Col<eT>::Col(const uword in_n_elem)
   {
   arma_extra_debug_sigprint();
   
-  #if (!defined(ARMA_DONT_ZERO_INIT))
+  if(arma_config::zero_init)
     {
     arma_extra_debug_print("Col::constructor: zeroing memory");
     arrayops::fill_zeros(Mat<eT>::memptr(), Mat<eT>::n_elem);
     }
-  #endif
   }
 
 
@@ -68,12 +69,11 @@ Col<eT>::Col(const uword in_n_rows, const uword in_n_cols)
   
   Mat<eT>::init_warm(in_n_rows, in_n_cols);
   
-  #if (!defined(ARMA_DONT_ZERO_INIT))
+  if(arma_config::zero_init)
     {
     arma_extra_debug_print("Col::constructor: zeroing memory");
     arrayops::fill_zeros(Mat<eT>::memptr(), Mat<eT>::n_elem);
     }
-  #endif
   }
 
 
@@ -87,12 +87,11 @@ Col<eT>::Col(const SizeMat& s)
   
   Mat<eT>::init_warm(s.n_rows, s.n_cols);
   
-  #if (!defined(ARMA_DONT_ZERO_INIT))
+  if(arma_config::zero_init)
     {
     arma_extra_debug_print("Col::constructor: zeroing memory");
     arrayops::fill_zeros(Mat<eT>::memptr(), Mat<eT>::n_elem);
     }
-  #endif
   }
 
 
@@ -312,10 +311,9 @@ Col<eT>::Col(const std::vector<eT>& x)
   {
   arma_extra_debug_sigprint_this(this);
   
-  if(x.size() > 0)
-    {
-    arrayops::copy( Mat<eT>::memptr(), &(x[0]), uword(x.size()) );
-    }
+  const uword N = uword(x.size());
+  
+  if(N > 0)  { arrayops::copy( Mat<eT>::memptr(), &(x[0]), N ); }
   }
   
   
@@ -328,12 +326,11 @@ Col<eT>::operator=(const std::vector<eT>& x)
   {
   arma_extra_debug_sigprint();
   
-  Mat<eT>::init_warm(uword(x.size()), 1);
+  const uword N = uword(x.size());
   
-  if(x.size() > 0)
-    {
-    arrayops::copy( Mat<eT>::memptr(), &(x[0]), uword(x.size()) );
-    }
+  Mat<eT>::init_warm(N, 1);
+  
+  if(N > 0)  { arrayops::copy( Mat<eT>::memptr(), &(x[0]), N ); }
   
   return *this;
   }
@@ -343,11 +340,13 @@ Col<eT>::operator=(const std::vector<eT>& x)
 template<typename eT>
 inline
 Col<eT>::Col(const std::initializer_list<eT>& list)
-  : Mat<eT>(arma_vec_indicator(), 1)
+  : Mat<eT>(arma_vec_indicator(), uword(list.size()), 1, 1)
   {
-  arma_extra_debug_sigprint();
+  arma_extra_debug_sigprint_this(this);
   
-  (*this).operator=(list);
+  const uword N = uword(list.size());
+  
+  if(N > 0)  { arrayops::copy( Mat<eT>::memptr(), list.begin(), N ); }
   }
 
 
@@ -359,14 +358,11 @@ Col<eT>::operator=(const std::initializer_list<eT>& list)
   {
   arma_extra_debug_sigprint();
   
-  Mat<eT> tmp(list);
+  const uword N = uword(list.size());
   
-  arma_debug_check( ((tmp.n_elem > 0) && (tmp.is_vec() == false)), "Mat::init(): requested size is not compatible with column vector layout" );
+  Mat<eT>::init_warm(N, 1);
   
-  access::rw(tmp.n_rows) = tmp.n_elem;
-  access::rw(tmp.n_cols) = 1;
-  
-  (*this).steal_mem(tmp);
+  if(N > 0)  { arrayops::copy( Mat<eT>::memptr(), list.begin(), N ); }
   
   return *this;
   }
@@ -422,15 +418,7 @@ Col<eT>::operator=(Col<eT>&& X)
   {
   arma_extra_debug_sigprint(arma_str::format("this = %x   X = %x") % this % &X);
   
-  (*this).steal_mem(X);
-  
-  if( (X.mem_state == 0) && (X.n_alloc <= arma_config::mat_prealloc) && (this != &X) )
-    {
-    access::rw(X.n_rows) = 0;
-    access::rw(X.n_cols) = 1;
-    access::rw(X.n_elem) = 0;
-    access::rw(X.mem)    = nullptr;
-    }
+  (*this).steal_mem(X, true);
   
   return *this;
   }
@@ -623,7 +611,7 @@ Col<eT>::operator=(const subview_cube<eT>& X)
 
 template<typename eT>
 inline
-arma_cold
+arma_deprecated
 mat_injector< Col<eT> >
 Col<eT>::operator<<(const eT val)
   {
@@ -634,6 +622,7 @@ Col<eT>::operator<<(const eT val)
 
 template<typename eT>
 arma_inline
+arma_warn_unused
 const Op<Col<eT>,op_htrans>
 Col<eT>::t() const
   {
@@ -644,6 +633,7 @@ Col<eT>::t() const
 
 template<typename eT>
 arma_inline
+arma_warn_unused
 const Op<Col<eT>,op_htrans>
 Col<eT>::ht() const
   {
@@ -654,6 +644,7 @@ Col<eT>::ht() const
 
 template<typename eT>
 arma_inline
+arma_warn_unused
 const Op<Col<eT>,op_strans>
 Col<eT>::st() const
   {
@@ -664,6 +655,7 @@ Col<eT>::st() const
 
 template<typename eT>
 arma_inline
+arma_warn_unused
 const Op<Col<eT>,op_strans>
 Col<eT>::as_row() const
   {
@@ -1062,15 +1054,28 @@ Col<eT>::shed_rows(const Base<uword, T1>& indices)
 
 
 
-//! insert N rows at the specified row position,
-//! optionally setting the elements of the inserted rows to zero
 template<typename eT>
+arma_deprecated
 inline
 void
 Col<eT>::insert_rows(const uword row_num, const uword N, const bool set_to_zero)
   {
   arma_extra_debug_sigprint();
   
+  arma_ignore(set_to_zero);
+  
+  (*this).insert_rows(row_num, N);
+  }
+
+
+
+template<typename eT>
+inline
+void
+Col<eT>::insert_rows(const uword row_num, const uword N)
+  {
+  arma_extra_debug_sigprint();
+  
   const uword t_n_rows = Mat<eT>::n_rows;
   
   const uword A_n_rows = row_num;
@@ -1079,30 +1084,26 @@ Col<eT>::insert_rows(const uword row_num, const uword N, const bool set_to_zero)
   // insertion at row_num == n_rows is in effect an append operation
   arma_debug_check_bounds( (row_num > t_n_rows), "Col::insert_rows(): index out of bounds" );
   
-  if(N > 0)
+  if(N == 0)  { return; }
+  
+  Col<eT> out(t_n_rows + N, arma_nozeros_indicator());
+  
+        eT* out_mem = out.memptr();
+  const eT*   t_mem = (*this).memptr();
+  
+  if(A_n_rows > 0)
     {
-    Col<eT> out(t_n_rows + N, arma_nozeros_indicator());
-    
-          eT* out_mem = out.memptr();
-    const eT*   t_mem = (*this).memptr();
-    
-    if(A_n_rows > 0)
-      {
-      arrayops::copy( out_mem, t_mem, A_n_rows );
-      }
-    
-    if(B_n_rows > 0)
-      {
-      arrayops::copy( &(out_mem[row_num + N]), &(t_mem[row_num]), B_n_rows );
-      }
-    
-    if(set_to_zero)
-      {
-      arrayops::inplace_set( &(out_mem[row_num]), eT(0), N );
-      }
-    
-    Mat<eT>::steal_mem(out);
+    arrayops::copy( out_mem, t_mem, A_n_rows );
     }
+  
+  if(B_n_rows > 0)
+    {
+    arrayops::copy( &(out_mem[row_num + N]), &(t_mem[row_num]), B_n_rows );
+    }
+  
+  arrayops::fill_zeros( &(out_mem[row_num]), N );
+  
+  Mat<eT>::steal_mem(out);
   }
 
 
@@ -1230,7 +1231,7 @@ Col<eT>::fixed<fixed_n_elem>::fixed()
   {
   arma_extra_debug_sigprint_this(this);
   
-  #if (!defined(ARMA_DONT_ZERO_INIT))
+  if(arma_config::zero_init)
     {
     arma_extra_debug_print("Col::fixed::constructor: zeroing memory");
     
@@ -1238,7 +1239,6 @@ Col<eT>::fixed<fixed_n_elem>::fixed()
     
     arrayops::inplace_set_fixed<eT,fixed_n_elem>( mem_use, eT(0) );
     }
-  #endif
   }
 
 
@@ -1579,6 +1579,7 @@ Col<eT>::fixed<fixed_n_elem>::operator=(const fixed<fixed_n_elem>& X)
 template<typename eT>
 template<uword fixed_n_elem>
 arma_inline
+arma_warn_unused
 const Op< typename Col<eT>::template fixed<fixed_n_elem>::Col_fixed_type, op_htrans >
 Col<eT>::fixed<fixed_n_elem>::t() const
   {
@@ -1590,6 +1591,7 @@ Col<eT>::fixed<fixed_n_elem>::t() const
 template<typename eT>
 template<uword fixed_n_elem>
 arma_inline
+arma_warn_unused
 const Op< typename Col<eT>::template fixed<fixed_n_elem>::Col_fixed_type, op_htrans >
 Col<eT>::fixed<fixed_n_elem>::ht() const
   {
@@ -1601,6 +1603,7 @@ Col<eT>::fixed<fixed_n_elem>::ht() const
 template<typename eT>
 template<uword fixed_n_elem>
 arma_inline
+arma_warn_unused
 const Op< typename Col<eT>::template fixed<fixed_n_elem>::Col_fixed_type, op_strans >
 Col<eT>::fixed<fixed_n_elem>::st() const
   {
@@ -1847,7 +1850,7 @@ Col<eT>::Col(const arma_fixed_indicator&, const uword in_n_elem, const eT* in_me
 
 
 
-#ifdef ARMA_EXTRA_COL_MEAT
+#if defined(ARMA_EXTRA_COL_MEAT)
   #include ARMA_INCFILE_WRAP(ARMA_EXTRA_COL_MEAT)
 #endif
 
